//Program for reading and displaying the temperature and humidity using
//the Sensirion SHT11 sensor and switching  outputs in relation to trip temperatures and humidity.
//The sensor uses a synchronous clock and data line MSB first.
//IButton 1904 for non volatile timekeeping
//Max 823 times out at 1.2s

//Copyright C.D. Barnard 1/4/2003 Qld. Australia

//Version 1.04
//Changes since v1.02
//Added external watchdog capability

//Version 1.03
//Changes since v1.02
//Changed humidity algorithm , error is now less than 1% (0.02 - 0.16%) apart from a dead band
//between 37 and 38% RH unadjusted, where the calculated error is 1%. 
//This is due to NOT using floating point maths.

//Version 1.02
//Changes since v1.01
//Disabled watchdog, timing out problem - from a safety view external watchdog preferable
//Added functions for Futoba M202SD08GL Vacuum Fluorescent Display in serial mode

//Version 1.01
//Changes since v1.00
//Watchdog timer enabled for safety reasons. If controller fails then watchdog will reset the controller

//Version  1.00
//Changes since v0.95
//Implimented PWM soft start/finish for low humidity and over temp trip outputs
//Altered Splash screen position to make it more central on LCD

//Version 0.95
//Changes since v.90
//Removed iButton time/day functions due to compiler problems
//added conv_delay in tasktime routine, this now counts the ~245mS conversion delay from sensor
//removed delay and delay3 from conversion delay and deleted
//now all functions take place within the ~13.1mS loop time
//made t_c (temperature ) and rh_adj (humidity) global variables to use in trip function
//Splash now CLI - MATE 1

//*************************************PORT PINOUTS**********************************************************
//                                     16F873                                                                                                  ***
//  					______     _______                                                                         ***
//   				      I        |_|      B7    I  LCD DB7                                                      ***
//                         EncoderB   I  A0             B6    I  LCD DB6                                                      ***
//                         EncoderA   I  A1             B5    I  LCD DB5                                                      ***
//                         PushButton |  A2             B4    I  LCD DB4  / VFD Rx                                       ***
//    			   Watchdog   |  A3             B3    I  LCD ENABLE  / VFD RST                              ***
//    			              |  A4             B2    I  LCD  RS  / VFD BUSY Pin                             ***
//   				      |  A5             B1    I                                                                      ***
// 				      |                 B0    I                                                                      ***
//			              I                       I                                                                      ***
//				      I                       I                                                                      ***
//				      I  C0             C7    I                                                                      ***
//                         Humidity   |  C1             C6    I                                                                      ***
// 	                   Over Temp  |  C2             C5    I  Under Temp output                                         ***
// 	                   SHT Clock  I  C3             C4    I  SHT Data                                                      ***
//                                       ________________                                                                                           ***
// ****************************************************************************************************************** 
//

//#define VFD
#ifndef VFD
	#define LCDPORT PORTB
	//LCD uses high nibble of port
	#define TOP
#endif
#include <P16F873.h>
//#include <P16F873_bits.h>
#include <P16F877_2_INC.h>
#include <portbits.h>
#include <strings.h>
#include <datalib.h>
#include <lcd.h>
#include "climate.h"

//#__config 0x3F72
//Note pin naming is from the VFD's point of view
//The test pin of VFD not currently used
//all pins outputs except BUSY which is input
#ifdef VFD
	#define CONTROL_PORT PORTB
	#define DATA_PORT    PORTB
	#define TEST_PIN bRB5
	#define RST_PIN  bRB3
	#define BUSY_PIN bRB2
	#define RXD_PIN  0x04
	#include <vfd.h>
#endif
#include <delays.h>

//Pulsewidth macro for sensor function
//could use CD macro instead
#pragma asm
	
	clockd equ D'1000'
	freq  equ (D'10000000000'/_APROCFREQ)*4
	cycle equ (clockd/freq)
	
	pulsewidth macro delayus
	  dly=delayus*cycle
	  while dly>0
	  	nop
	  	dly-=1
	  endw
	endm
#asmend

//LCD pin setups
#ifndef VFD
	const int rsbit=0x02;
	const int ebit=0x03;
	const unsigned char RSPORT=&PORTB;
	const unsigned char EPORT=&PORTB;
#endif

#ifdef VFD
	const uchar SERIALPORT_OUT=&PORTB;
#endif

//Sensor and Button pins and ports
const unsigned char SHT_data=0x04;		//Data line on bit 4
const unsigned char SHT_clock=0x03;		//Sensor clock on bit 3
const unsigned char enc_A=0x01;			//Encoder A channel on bit 1
const unsigned char enc_B=0x00;			//Encoder B channel on bit 0
const unsigned char sensor_out=&PORTC;		//Sensor on PORTC



void main()
{
	
	count=200;
	setup();
	
	#ifndef VFD
		disp_init(initLCD);
	#else
		VFDreset();		//Hardware reset, takes VFD reset pin low 12uS
		VFD_send(CUROFF); 	//No cursor display
	#endif

	while(count)
	 {
	    display(splash);			//display  project name 
	    tasktime();				//for 3 seconds
	    count--;
	 }

	SHT_reset();				//Wake sensor

	#ifndef VFD
		disp_init(clear);		//clear the LCD of characters
	#else
		VFD_send(RST);
		VFD_send(CUROFF);	
		VFD_send(DIMM);
		VFD_send(DIM4);			//Dim display by 40%
	#endif

	conv_delay=20;				//initialise sensor conversion time
	TRISA=0x07;				//Change (A:3) to an output WDT now enabled
	//Main loop takes approx 13mS. WDT is a nominal 1.2seconds.
	//WD is taken high at top of loop and taken low	at every other function call
	//This means that if we get stuck in a function a reset is guaranteed			
	while(1)
	{	
	  WDT=1;				//Max 6303  or 823  watchdog switch high at start of loop
	  keypress();				//Test for button press
	  domenu();				//Perform button choice
	  WDT=0;
	  SHT_measureTemp(&sht);		//get temperature every 3.5S
	  WDT=1;
	  SHT_hum_measure(&sht);		//get humidity every 3.5S
	  WDT=0;
	  trip_check(&trip);			//Check trip points
	  WDT=0;				//Pulse WDT low if we don't reach here, reset
	  tasktime();				//loop time
     	}
}


void setup()
{
	ADCON1=0x07;				//switch adc off
 	PORTA=0x00;					
 	#ifndef VFD
		LCDPORT=0x00;
	#else
		CONTROL_PORT=0x08;
	#endif
 	PORTC=0x10;				//SHT Data line idle high, clock idle low
 	TRISA=0x0F;				//Bit 3 input so that WDT is disabled at this point
 	#ifndef VFD
		TRISB=0x00;
 	#else
		TRISB=0x04;
	#endif
	TRISC=0x00;	
 	PR2=31;					//Set period for PWM to  19.531KHz @ 10MHz
 	CCPR1L=0;	   			//Duty cycle1 to zero (PWM output low)
 	CCPR2L=0;				//Duty cycle2 to zero
 	T2CON=0x05;				//Prescale by 4 Timer2 on
 	OPTION_REG=(1<<PS2)|(1<<PS1)|(PS0); 	// to divide by 128for task time of 13.1mS
 	sysflag.SHT_err=1; 			//Start with error flag set
	old_enc_state=1;
	buttonhold=0;
	sysflag.oldpress=0;
	scale='C';				//Temperature scale defaults to centigrade
	duty1=0;
	duty2=0;
}

//Loop time function set to 13.1mS with 10MHz freq
void tasktime()
{
	
	while(!(INTCON&(1<<T0IF)));		//check for timer0 flag
	B_INTCON.T0IF=0;
	conv_delay--;
	
}

void fault(unsigned char eMode)
{
	count=75;
	while(count)
	 {	
	   display(error0);
	
	   switch (eMode)
	    {	
		case SHT_err : display(error1);
				   break;
		//case iBt_err : display(error2);
		default : break;
	    }
	  tasktime();
	  count--;
	 }
	#ifndef VFD
		disp_init(clear);
	#else
		VFD_send(RST);			//resets VFD and clears screen
		VFD_send(CUROFF);
	#endif
}

//SHT_reset function
//This function wakes the sensor from sleep. 
//The sequence is with data line high, clock the SCL line 9 times then call 
//transmission start sequence.
 
void SHT_reset()
{
 	//#pragma asmfunc SHT_reset
 	#asm
 	    clrf count
 	    bsf _sensor_out,_SHT_data		;ensure data bit is high
 	    bsf _sensor_out,_SHT_clock  	;bring clock high
 	    nop
 	    nop
 loop2 	bcf _sensor_out,_SHT_clock		;clock low
 	    bsf _sensor_out,_SHT_clock		;high again
 	    incf count				;increment loop count
 	    btfss count,3			;have we done this 8 times
 	    goto loop2
 	    nop
        bcf _sensor_out,_SHT_clock	 	;bring clock low to idle
        call SHT_start			;call transmission start sequence
	#asmend
	
}

//Function SHT_start
//This in the sensor document is the transmission start sequence
//With the data line high, the clock is brought high and the data line brought low
//the SCL line is clocked again and the data line is then brought high ready for
//data transmission or reception

void SHT_start()
{
  //#pragma asmfunc SHT_start
  #asm
	  bsf _sensor_out,_SHT_data
	  bcf _sensor_out,_SHT_clock
	  nop
	  bsf _sensor_out,_SHT_clock
	  nop
	  bcf _sensor_out,_SHT_data
	  nop
	  bcf _sensor_out,_SHT_clock
	  pulsewidth 4
	  bsf _sensor_out,_SHT_clock
	  nop
	  bsf _sensor_out,_SHT_data
	  nop
	  bcf _sensor_out,_SHT_clock
  #asmend

}

//Function SHT_write
//enter this function with command to sensor in d_byte
//Data line high and clock low

void SHT_write(unsigned char d_byte)
{
	unsigned char loop;
	
	for(loop=0x80;loop>0;loop>>=1) 			//data out MSB first
	{
	 	if (loop & d_byte)
	 	  {
	 	      SHT_bDATA=1;
	 	  }
	 	else
	 	 {
	 	      SHT_bDATA=0;
	 	 }
	 	   #asm
	 	 	 bsf _sensor_out,_SHT_clock
	 	   	 pulsewidth 4
	 	   	 bcf _sensor_out,_SHT_clock
	 	   #asmend
	}
	#asm
		bsf STATUS,RP0
		bsf TRISC,_SHT_data			;Make pin an input
		bcf STATUS,RP0
		bsf _sensor_out,_SHT_clock
		btfss _sensor_out,_SHT_data 		;Wait for acknowledge=low
		bcf sysflag,_SHT_err			;Acknowledge received clear flag
		bcf _sensor_out,_SHT_clock
	#asmend
	if (sysflag&(1<<SHT_err))
	 {
	    fault(SHT_err);			//No acknowledge sensor faulty or missing
	 }
}


//Function SHT_read.
//This function reads 3 bytes MSB first. The first two bytes are the conversion data and the 3rd byte is a  CRC-8
//checksum.  The first two bytes require an ack low to be sent to the sensor. The CRC byte has no ack to finish
//transmission. Sensor goes into sleep mode.
//Ack is sent or not on the 9th clock bit
//data is returned via pointer to array sht.

void SHT_read()
{
	unsigned char getbytes=3;
	
	sysflag.SHT_ack=1;			//We need initial acknowledge on first 2 bytes
	#asm
    	movlw sht+1				;Address of second element of array
	movwf pSht				;Pointer now holds the address
	#asmend
	while (getbytes)	//we need to get 3 bytes of data
	{
	#asm
			
		movf pSht,W			;get pointer to array
		movwf FSR			;move it to FSR
		clrf count			;set bit counter to zero
		clrf indata			;make sure data register is cleared 
		bcf STATUS,C			;clear the Carry ready for incoming data
	loop1	bsf _sensor_out,_SHT_clock
		btfsc _sensor_out,_SHT_data 	;if data pin is low leave indata current bit at zero
		bsf STATUS,C			;else data pin was high
		rlf indata,F			;fill indata MSB first			
		incf count			;bit counter
		bcf _sensor_out,_SHT_clock
		btfss count,3			;have we done 8 bits of data?
		goto loop1
		movf indata,W			;Yes so ready data to put into			
		movwf INDF			;the current array position
		decf pSht,F		 	;move pointer to next position in array
		btfss sysflag,_SHT_ack		;if ack==1
		goto noack			;if ack==0
		bsf STATUS,RP0
		bcf TRISC,_SHT_data		;Make pin an output
		bcf STATUS,RP0
		bcf _sensor_out,_SHT_data	;bring data line low to ack
		bsf STATUS,RP0
		bsf TRISC,_SHT_data		;Make pin an input
		bcf STATUS,RP0
	noack: 	bsf _sensor_out,_SHT_clock
	 	pulsewidth 4
	 	bcf _sensor_out,_SHT_clock
	#asmend
	
		if (--getbytes==1);
		{
		  sysflag.SHT_ack=0;		//No acknowledge on last byte
		  #asmline movlw sht+2
		  #asmline movwf pSht
		}
		
    }
    
    #asm
    	bsf _sensor_out,_SHT_data		;idle high
    	bsf STATUS,RP0
    	bcf TRISC,_SHT_data			;return pin to output
    	bcf STATUS,RP0
	#asmend
}

//Function SHT_measureTemp
//Approximately every 3 seconds sends read TEMP command to sensor and converts sensor data to temperature reading
//in either Centigrade or Fahrenheight.

void SHT_measureTemp(int *temp)
{
  
  int temp_ticks;
  uchar d1,d2;
     
 while (delay2)
 {
  delay2--; 	 				//make measurement every 3.5 seconds
  return;
 }
 
  if(!(sysflag&(1<<done)))
    {
  	SHT_start();
  	SHT_write(TEMP);
  	sysflag.done=1;			//on next entry after task loop check for conversion
  	conv_delay=20;			//initialise timer counter
    }
  	
 if (!(conv_delay))				//wait for conversion time,  approx 245mS
    {
 		
  	if(SHT_bDATA)			//If data line not pulled low by sensor to signal data ready
  	  {
  	      sysflag.SHT_err=1;
  	      fault(SHT_err);
 	      return;
  	  } 
 	SHT_read();				//read in the data
 	temp_ticks=*temp;
 		
 	if (sysflag&(1<<Scale))
 	   {
  	       d2=56;				//Fahrenheight scale, dividing by 56 and
  	       d1=39;		//deducting 39 is approx equal to dividing  sensor ticks by 1.8
 	   }
 	else
  	   {
  	       d2=100;				 //Centigrade reading
  	       d1=40;			//Taken from data sheet d2=0.01 and d1= -40
 	   }
 		 		
 	temp_ticks/=d2;			//Divide raw sensor data by  D2 sensor constant
 	 
 	if (temp_ticks<d1)		//Check for minus temperatures
  	   {
    	       t_c=D1-temp_ticks;	//adjust back to a positive number
   	       sysflag.T_sign=1;	//signal this number should have a minus sign
   	   }
   	else
  	   {
  		t_c=temp_ticks-d1;
  	   }
  		 
  				 	
 	*temp=t_c;
 	template[0]=COLUMN3;		//Line one column 3
 	disp_num(template,t_c,3,BLANK); 	// 3 digits no leading zero
 	#ifdef VFD
 	    template[4]=0xEF;		//code for degree symbol on VFD display
 	#else
 	    template[4]=0xDF;		//insert degree symbol
 	#endif
 	    template[5]=scale;
 	    if (sysflag&(1<<T_sign))
 		 template[1]='-';		//test for neg temp
 		
 	display(template);
 	delay2=252;
 	sysflag.done=0;			//finished conversion, next time round transmission start
    }
  
}

//Function SHT_hum_measure
//this function sends a read HUMID request to the SHTsensor and converts the data into a true humidity reading.
//The true humidity is obtained from the relative humidity data factoring in the difference in temperature from the 
//base line measurement of 25 deg C

void SHT_hum_measure(int *p_humidity)
{
  ulong rh_ticks;
  ulong rh_lin;
  uint temp ;
  uchar base_t;
  
 
   while (delay1)
    {
      delay1--; 	 		//make measurement every 3.5 seconds
      return;
    } 
    
  
   if(!(sysflag&(1<<done)))
    {
  	SHT_start();
  	SHT_write(HUMID);
  	sysflag.done=1;
  	conv_delay=20;				//initialise timer counter
    }
 			
  if (!(conv_delay))			//wait for conversion time, approx 245mS
   {
      
      if(SHT_bDATA) 			//If data line not pulled low by sensor to signal data ready
       {
         sysflag.SHT_err=1;
         fault(SHT_err);
 	 return;
       }
       
      SHT_read();
  		
      if(sysflag&(1<<Scale))
       {
  	 base_t=77;			//Referenced to 25 centigrade. 77f = 25c
       }
      else
       {
  	 base_t=25;
       }
  	 
      rh_ticks=*p_humidity;
      if(rh_ticks <1801)	//1800 ticks= approx 60% humidity unadjusted
       {
         rh_lin=((rh_ticks*1000)/30960)+2; //multiply by 1000 to avoid fractions
       }
      else
       {
         rh_lin=((rh_ticks*1000)/31046);
       }
       
      if (t_c > base_t)			//adjust humidity reading against base temperature of 25c
       {
 	  temp=(t_c-base_t)*(rh_lin*26/100);
 	  temp/=100;
 	  rh_adj= temp + rh_lin;
       }
      else
       {
          rh_adj=rh_lin;
       }
      
      if (rh_adj>99)
       {
          rh_adj=100;
       }	
     
      *p_humidity=rh_adj;
      template[0]=COLUMN10;
      disp_num(template,rh_adj,3,BLANK);
      template[4]='%';
      template[5]='H';
      display(template);
      delay1=252;
      sysflag.done=0;
   }
}

//Function trip_check
//Compares the trip settings of Temperature (over and under), and humidity against those set in the DoMenu function.
//Humidty and OverTemp  switch on their respective PWM modules and operate as a soft start feature. Starting from a 
//duty cycle of zero up to 100% over a period of 1 second, they also ramp down at switch off.
//Under temperature operates as a pure switch.
//The PWM frequency is 19625KHz.

void trip_check(trips *pTrip)
{
 	if (rh_adj < pTrip->humidity )
 	  {					//if current humidity < trip humidity switch on with soft start
 		CCP1CON=0x2C;				//PWM module1 on, output pin RC2
 					
 		if (++duty1 > 30)
 		 {
 		    duty1=31;
 		 }
 		CCPR1L=duty1;
 	  }
 	else
 	  {
 		if(!(duty1 == 0))
 		   duty1--;
 		CCPR1L=duty1;
 		#asm
 		  movf duty1,W
 		  btfsc STATUS,Z	;Is duty zero?
 		  movwf CCP1CON		;Yes so turn CCP1 off	
 		#asmend
 	   }
 	
 	if (t_c < pTrip->undertemp ) 
 		bRC5=1;						//if current temp < undertemp switch heater on
 	else
 		bRC5=0;	
 	
 	if (pTrip->overtemp > pTrip->undertemp)
 	 {
 	 	if (t_c > pTrip->overtemp)		//if current temp is > overtemp  switch on
 		 {
 		   CCP2CON=0x2C;				//PWM module2 on, output pin RC1, 
 		   if (++duty2 > 30)
 		    {
 		    duty2=31;
 		    }
 		CCPR2L=duty2;
 		 }
 		 else
 		 {
 		   if(!(duty2 == 0))
 		    duty2--;
 		   CCPR2L=duty2;
 		 #asm
 		  movf duty2,W
 		  btfsc STATUS,Z	;Is duty zero?
 		  movwf CCP2CON		;Yes so turn CCP2 off	
 		#asmend
 		 }
 	 }
 	//else
 	 
 	 /*{
 	   CCPR2L=0;
 	   //CCP1CON=0;
 	   //CCP2CON=0;
 	 }*/						//otherwise off
 	 
   

}

//Function getenc
//Checks the rotary encoder for increment or decrement. This function checks for the sequence 01>11 for an increment and
//11>10 for a decrement setting the flag enc_state at 1 for + and -1 (255) for -.
 
void getenc()
{
	
	//encoder channel B=0
	if (!bRA0)
	  old_enc_state=1;			//show that we've got here
	
	if ((bRA0) && (old_enc_state))		//if Channel B now =1
	 {
	   if (bRA1)				//if encoder now =11 ,we're going CW so increment
	    {	
	       enc_state=1;			//set enc_stateto show
	       old_enc_state=0;			//set to zero, wait until this event again
	    }
	   
           if((!bRA1) && (old_enc_state))	//Channel A=0 and B =1 going CCW
	    {
	       enc_state=-1;
	       old_enc_state=0;
	    }
		
	 }
	else
	 {
	   enc_state=0;				//encoder hasn't moved
	 }
			
		
			
	
}

//Function keypress
//detects the pressing of the integral pushbutton on the encoder. discriminates between short button pushes and long (approx 1 second)

void keypress()
{
	sysflag.lpress=0;
	sysflag.spress=0;
	
	//detect button press
	if (PB)
	   sysflag.keydown=1;
	else
	   sysflag.keydown=0;
		
	//low to high start timing for button press length
	
	if (!(sysflag&(1<<oldpress)))
	 {
	   if(sysflag&(1<<keydown))
	   buttonhold=38;		//threshold to discriminate between long or short press
	 }
	
	//detect long key press
	
	if (sysflag&(1<<keydown))
	 {
	   if (buttonhold==0)
	    if(!(sysflag&(1<<keydone)))
	     {
	 	sysflag.lpress=1;
	 	//sysflag.spress=0;
	 	sysflag.keydone=1;
	     }
	  }
	else
	   sysflag.keydone=0;
					 
	//detect short button press		
	
	if (sysflag&(1<<oldpress))
	
	  if (!(sysflag&(1<<keydown)))
	   {
	      if (buttonhold)
	       {
	          sysflag.spress=1;
	          sysflag.keydone=0;
	          buttonhold=0;
	       }
	   }
					
	if (buttonhold)
	  buttonhold--;
	
	if (sysflag&(1<<keydown))
	   sysflag.oldpress=1;		//last key state was high _|------
	else
	   sysflag.oldpress=0;		//last key state was low   ----|__	
	 

}

//Function domenu
//This function is used to convert the pushbutton press length into short press - show trip settings and
//long press set trip settings and formats the display template ready for output to either n LCD or VFD

#ifndef VFD
void domenu()
{
	
	
	if (sysflag&(1<<Scale))
	  max_trip=253;				//Fahrenheight max temp 253 deg
	else 
	  max_trip=123;				//Centigrade max temp 123 deg
	
	//short button press
	if (sysflag&(1<<spress))
	 {
	   lmenu=0;				//Can't be in two menus at once!
	   display(clear2);
	   if (++menu==(MAX_MENU))
	    {
	       menu=0;
	       return;
	    }
	   
	  }
	
	
	   //Show Humidity trip
	if (menu==2)
	 {	
	 	strcpy(template+1,tripscrn[menu]);
	    template[3]='=';
	    template[4]=0;
	    template[0]=0xC1;
	    display(template);	
	    display1(trip.humidity,3,BLANK);
	 }

	//Set month number 1=Jan, 12=Dec	
	if (menu==3)
	 {
		strcpy(template+1,tripscrn[menu]);
	    template[3]='=';
	    template[4]=0;
	    template[0]=0xC1;
	    display(template);	
	    display1(trip.undertemp,3,BLANK);	
	 }
	
	//Day number in month
	if (menu==4)
	 {	
		strcpy(template+1,tripscrn[menu]);
	    template[3]='=';
	    template[4]=0;
	    template[0]=0xC1;
	    display(template);	
	    display1(trip.overtemp,3,BLANK);
	 }
	
	
	
	//Long button press
	if (sysflag&(1<<lpress))
	 {
	    menu=0;				//make sure only this selection valid
	    //enc_count=0;
	    display(clear2);
	    if (++lmenu==MAX_MENU)		//this menu has a maximum of four choices
	     {
		lmenu=1;
		//return;
	     }
	    
	    strcpy(template+1,tripscrn[lmenu]);
	    template[0]=0xC1;
	    display(template);
	 }
	
	if (lmenu==1)				//long press menu selection
	 {
	   getenc();
	   if (enc_state==1)
	    {
	       sysflag.Scale=1;		 	//flag for fahrenheit
	       scale='F';
	    }
	  
	   if (enc_state==-1)
	    {
	      sysflag.Scale=0; 			//flag for centigrade
	      scale='C';
	    }
	   template[0]=0xC5;
	   template[1]=scale;
	   template[2]=0;
	   display(template);
	 }

	//asm routine saves 6 program words over  using if <1 || > 99 construct	
	if (lmenu==2)		//set under humidity trip point
	 {	
	    getenc();
	    if(enc_state==1)
	     {
		#asmline incf trip,F
		
	     }
				
	    if(enc_state== -1)
	     {
		#asmline decf trip,F
		
	     }
				
		
	    if (trip.humidity<1 || trip.humidity>100)
	       trip.humidity=50;
	    display1(trip.humidity,3,BLANK);
	 }
		
   	if (lmenu==3)		//set under temperature trip point
   	 {
	    getenc();
	    if(enc_state==1)
	     {
		#asmline incf trip+1,F
	     }
				
	    if(enc_state== -1)
	     {
		#asmline decf trip+1,F
	     }
				
	    //	if (trip.undertemp==max_trip)
	     {
		#asm
			movf max_trip,W
			sublw 255
			addwf trip+1,W
			btfsc STATUS,C
			goto divide
			goto end1
	divide:	bcf STATUS,C
			rrf trip+1,F
			bcf STATUS,C	;require due to fahrenheight measurement	
			rrf trip+1,F	;This yields 63 if scale in F or 31 if in C
	end1:
		#asmend
	     }
	    display1(trip.undertemp,3,BLANK);
			
	 }
		
       if (lmenu==4)	//set over temperature trip point. Can be no more than 10 deg over undertrip temp
        {	
	
          getenc();
          if(enc_state==1)
           {
	      #asmline incf trip+2,F
           }

         if(enc_state== -1)
          {
	     #asmline decf trip+2,F
          }
				
         //if (trip.overtemp==max_trip)//check this is no more than max temperature 
         #asm
	       movf max_trip,W
	       sublw 255,W
	       addwf trip+2,W
	       btfsc STATUS,C
	       clrf trip+2			;zero trip.overtemp if < 0 or > max_temp
        #asmend
        if (trip.overtemp==(trip.undertemp+11))//check limit
	       trip.overtemp=trip.undertemp+5;	//start again from halfway mark
        display1(trip.overtemp,3,BLANK);
      }	

}
#endif

#ifdef VFD
void domenu()
{
	
	
	if (sysflag&(1<<Scale))
	  max_trip=253;				//Fahrenheight max temp 253 deg
	else 
	  max_trip=123;				//Centigrade max temp 123 deg
	
	//short button press
	if (sysflag&(1<<spress))
	 {
	   lmenu=0;				//Can't be in two menus at once!
	   //display(clear2);
	   VFD_send(LF);
	   if (++menu==(MAX_MENU))
	    {
	       menu=0;
	       return;
	    }
	   
	  }
	
	
	   //Show Humidity trip
	if (menu==2)
	 {	
	 	strcpy(template+1,tripscrn[menu]);
	    template[3]='=';
	    template[4]=0;
	    template[0]=0x01;
	    display(template);	
	    display1(trip.humidity,3,BLANK);
	 }

	//Set month number 1=Jan, 12=Dec	
	if (menu==3)
	 {
		strcpy(template+1,tripscrn[menu]);
	    template[3]='=';
	    template[4]=0;
	    template[0]=0x01;
	    display(template);	
	    display1(trip.undertemp,3,BLANK);	
	 }
	
	//Day number in month
	if (menu==4)
	 {	
		strcpy(template+1,tripscrn[menu]);
	    template[3]='=';
	    template[4]=0;
	    template[0]=0x01;
	    display(template);	
	    display1(trip.overtemp,3,BLANK);
	 }
	
	
	
	//Long button press
	if (sysflag&(1<<lpress))
	 {
	    menu=0;				//make sure only this selection valid
	    //enc_count=0;
	    //display(clear2);
	    VFD_send(LF);
	    if (++lmenu==MAX_MENU)		//this menu has a maximum of four choices
	     {
		lmenu=1;
		//return;
	     }
	    
	    strcpy(template+1,tripscrn[lmenu]);
	    template[0]=0x01;
	    display(template);
	 }
	
	if (lmenu==1)				//long press menu selection
	 {
	   getenc();
	   if (enc_state==1)
	    {
	       sysflag.Scale=1;		 	//flag for fahrenheit
	       scale='F';
	    }
	  
	   if (enc_state==-1)
	    {
	      sysflag.Scale=0; 			//flag for centigrade
	      scale='C';
	    }
	   template[0]=0x05;
	   template[1]=scale;
	   template[2]=0;
	   display(template);
	 }

	//asm routine saves 6 program words over  using if <1 || > 99 construct	
	if (lmenu==2)		//set under humidity trip point
	 {	
	    getenc();
	    if(enc_state==1)
	     {
		#asmline incf trip,F
		
	     }
				
	    if(enc_state== -1)
	     {
		#asmline decf trip,F
		
	     }
				
		
	    if (trip.humidity<1 || trip.humidity>100)
	       trip.humidity=50;
	    display1(trip.humidity,3,BLANK);
	 }
		
   	if (lmenu==3)		//set under temperature trip point
   	 {
	    getenc();
	    if(enc_state==1)
	     {
		#asmline incf trip+1,F
	     }
				
	    if(enc_state== -1)
	     {
		#asmline decf trip+1,F
	     }
				
	    //	if (trip.undertemp==max_trip)
	     {
		#asm
			movf max_trip,W
			sublw 255
			addwf trip+1,W
			btfsc STATUS,C
			goto divide
			goto end1
	divide:	bcf STATUS,C
			rrf trip+1,F
			bcf STATUS,C	;require due to fahrenheight measurment	
			rrf trip+1,F	;This yields 63 if scale in F or 31 if in C
	end1:
		#asmend
	     }
	    display1(trip.undertemp,3,BLANK);
			
	 }
		
       if (lmenu==4)	//set over temperature trip point. Can be no more than 10 deg over undertrip temp
        {	
	
          getenc();
          if(enc_state==1)
           {
	      #asmline incf trip+2,F
           }

         if(enc_state== -1)
          {
	     #asmline decf trip+2,F
          }
				
         //if (trip.overtemp==max_trip)//check this is no more than max temperature 
         #asm
	       movf max_trip,W
	       sublw 255,W
	       addwf trip+2,W
	       btfsc STATUS,C
	       clrf trip+2			;zero trip.overtemp if < 0 or > max_temp
        #asmend
        if (trip.overtemp==(trip.undertemp+11))//check limit
	       trip.overtemp=trip.undertemp+5;	//start again from halfway mark
        display1(trip.overtemp,3,BLANK);
      }	

}
#endif

//Function display1
//This function takes the value to be displayed - the digit width and a leading zero or not indicator
//and formats them for output on the display.

#ifndef VFD
void display1(uchar value, uchar dig, uchar lz)
{
	disp_num(template,value,dig,lz);
	template[0]=0xC5;
	template[4]=0;
	display(template);
}
#endif

#ifdef VFD
void display1(uchar value, uchar dig, uchar lz)
{
	disp_num(template,value,dig,lz);
	template[0]=0x05;
	template[4]=0;
	VFD_send(LF);		//move to second line
	display(template);
}
#endif

//This libary file moved here, to save programming space and to enable simulator tracking

//Library file for LCD and VFD displays that are HD44780 compliant
//
//Copyright C D Barnard 12/01/2003
//V1.02
//29/8/2003 integrated Futoba M202SD08GL serial VFD functions with LCD file
//V1.01
//12/3/2003 amended  init wait time from 40uS to 1mS to cover all LCD's

//#include <P16F873.h>

//#include <delays.h>

//****************************************************************


#ifdef _disp_init
void disp_init(unsigned char *tmpPtr)
{
   


   unsigned char currentCH;
   unsigned char delay=2;  
   rs=0;
   while(delay)			//Delay 26mS to enable LCD to initialise
   {
   		
   		tasktime();
   		delay--;
   }
  #ifdef TOP
  while(*tmpPtr)
 	{
    	
    	currentCH=*tmpPtr;
    	LCDPORT&=0x0F;
    	LCDPORT=(currentCH&0xF0);
    	sendit(rs);
    	currentCH<<=4;
    	LCDPORT=currentCH;
    	sendit(rs);
    	Wait(1);
  	 	tmpPtr++;
 	} 
  #else
  while(*tmpPtr)
	{
    	
    	currentCH=*tmpPtr;
    	LCDPORT&=0xF0;
    	LCDPORT=(currentCH&0x0F);
    	sendit(rs);
    	currentCH>>=4;
    	LCDPORT=currentCH;
    	sendit(rs);
    	//t40();
    	Wait(1);
  	    tmpPtr++;
 	} 
 #endif
}
#endif

#ifdef _VFDreset
void VFDreset()
{
  RST_PIN=LOW;			//Take reset pin low to awake VFD
  #asm
    movlw _VFD_DELAY
    call Delay0
  #asmend 
  RST_PIN=HIGH;			//Return reset pin to operating state
}
#endif

#ifdef VFD
#ifdef _display
void display(uchar *tmpPtr)
{
  unsigned char currentCH;
  VFD_send(DP);
  while(*tmpPtr)		//while not end of string
  {
  	currentCH=*tmpPtr;
  	VFD_send(currentCH);
  	tmpPtr++;
  }

}
#endif

#else

#ifdef _display
void display(unsigned char *tmpPtr)
{
	
  unsigned char currentCH;
  
  rs=0;					//RS in Command mode for position byte
  
  #ifdef TOP
  while(*tmpPtr)		//LCD Data on bits D4-D7 of PORT
    {
    	
    	currentCH=*tmpPtr;
    	LCDPORT&=0x0F;				
    	LCDPORT=(currentCH&0xF0);	//Mask off upper nibble
    	sendit(rs);
    	currentCH<<=4;				//Now select lower nibble
    	LCDPORT=currentCH;
    	sendit(rs);
    	t40();//delay1W(40);
  		rs=1;						//make sure RS is in data mode
    	tmpPtr++;					//get next character
    }
  #else				
  while(*tmpPtr)	//LCD Data on bits D0->D3 of PORT
    {
    	//#asmline clrwdt
    	currentCH=*tmpPtr;
    	LCDPORT&=0xF0;
    	LCDPORT=(currentCH&0x0F);
    	sendit(rs);
    	currentCH>>=4;
    	LCDPORT=currentCH;
    	sendit(rs);
    	t40();
  	    tmpPtr++;
 	} 
 #endif
 }
#endif 
#endif

#ifndef VFD 
void sendit(unsigned char command)
{
    unsigned char bitmask;
	
  
  if(command&1)
   {
     
    #asmline bsf _RSPORT,_rsbit
   }
 else
   {
    #asmline bcf _RSPORT,_rsbit
   } 
 //EPORT^=1<<ebit;
 #asm
 	 bsf _EPORT,_ebit
	 nop
	 bcf _EPORT,_ebit
 #asmend
 //EPORT^=1<<ebit;
}
#endif

#ifdef _VFD_send
void VFD_send(uchar text)
{
 
 	while(BUSY_PIN)
 		{
 		}
	pSerialOut(text);
 
}
#endif

//This function provides a 40uS delay @ clock frequency of 10MHz
//alter count= to 5 for a 4MHz clock.

void t40()        
{
  unsigned char count=13;
  
  while(count)
  count--;
}


//Takes a 8 bit binary value and converts it to ASCII and holds it in
//an array ready for display.
#ifdef _disp_byte
void disp_byte(unsigned char *tmpPtr, unsigned char file)
{	unsigned char loop,btemp;
	
	loop=8;
	
	while (loop)
		{
			btemp=(file & 0x01);	//get value of bit0
			btemp|=0x30;			//Add 0x30 to make it ASCII
			tmpPtr[loop]=btemp;		//Fill the array (right to left)
			file>>=1;				//Shift byte one step to the right
			loop--;
		}
	tmpPtr[10]=0;					//EOF symbol
}
#endif

//Compute line and column number for character to be displayed.

#ifdef _disp_pos	
unsigned char disp_pos(unsigned char line, unsigned char column)
{	//Upto 4*16 display. Alter 0x90->0x94 and 0xD0->0xD4 for 4*20 display
	const unsigned char row[4]={0x80,0xC0,0x90,0xD0};
	
	line-=1;		//Adjust line to account for human notation
	line=row[line]+column;
	
	return line;

}

#endif

//Split number into individual digits and insert in correct position in array
//Also format for leading zero or space to be displayed.

#ifdef _disp_num
void disp_num(unsigned char *disp,unsigned int conv,unsigned char dig, unsigned char lz)
{
	const int digit_pos[]={1000,100,10,1};
	unsigned char pos,digit=0;
	unsigned char divby;
	pos=0;
	
	for (divby=(4-dig);divby < 3;divby++)
	{
		while(conv >= digit_pos[divby])
			{
				conv-=digit_pos[divby];	//deduct value of const int from number
				digit+=1;				//make digit equal number of subtractions
			}
		disp[pos+1]=num2hex(digit);		//assign it to correct position in array
		if (lz && disp[1]=='0')
			disp[pos+1]=0x20;			//If no leading zero make it a space
		digit=0;						//Clear digit ready for next
		pos++;
		#asmline clrwdt
	}
	digit=conv;							//units
	disp[pos+1]=num2hex(digit);	
	disp[pos+dig]=0;						//insert EOF at template position 6 in array
}
#endif

//Turn decimal into an ASCII number.

#ifdef _num2hex
unsigned char num2hex (unsigned char decimal)      
{
   unsigned char temp;
   
   if(decimal <10)
     {
       temp=decimal+0x30;	//Number less than 10 just add '0'
     }
   else
     {
       temp=decimal-10;
       temp+=0x41;			//number is between A and F 
     }
    
   return (temp); 
    
}
#endif



  
